////////////////////////////////////////////////////////////////////////
/*! \file   phong_geometry.glsl
 *  \brief  Filter triangle to display just one.
 *  \author jD Gascuel.
 *  \sa phong_vertex.glsl, phong_fragment.glsl
 */
////////////////////////////////////////////////////////////////////////
// $Id: cube_map_layers_geometry.glsl,v 1.1 2007/12/06 12:33:15 gascuel Exp $
// File History:
//   2007-01-08 : Creation.

#version 120
#extension EXT_geometry_shader4 : require

// Per rendering data :
uniform int uPrimitiveFilter = -1;  // Retain only one triangle if >= 0.

// For depth and debug mode
varying in  vec4 iP[];
varying out vec4 oP;

// For per-pixel-lighting:
varying in  vec3 iN[];				// Normal, output from vertex shader
varying out vec3 oN;				// Send to the fragment shader.

// Shadow map data :
varying in  vec4 iS[];				// Position, in light[1] frame.
varying out vec4 oS;

////////////////////////////////////////////////////////////////////////

#if defined(COLOR_TEXTURE)
#define COPY_TEXTURE(v)     gl_TexCoord[COLOR_TEXTURE] = gl_TexCoordIn[v][COLOR_TEXTURE]
#else
#define COPY_TEXTURE(v)
#endif

#if !defined(PER_PIXEL_LIGHTINGS)
#define COPY_COLOR(v)       gl_FrontColor = gl_BackColor = gl_FrontColorIn[v]
#else
#define COPY_COLOR(v)
#endif

#ifdef PER_PIXEL_LIGHTINGS
#define COPY_NORMAL(v)      oN = iN[v]
#else
#define COPY_NORMAL(v)
#endif

#if defined(PER_PIXEL_LIGHTINGS) && defined(SHADOW_MAPPING)
#define	COPY_L1_COORD(v)	oS = iS[v]
#else
#define	COPY_L1_COORD(v)
#endif

#if defined(SUBMODE_DEPTH) || defined(SUBMODE_DEBUG) || defined(PER_PIXEL_LIGHTINGS)
#define COPY_POSITION(v)    oP = iP[v]; gl_Position = gl_PositionIn[v]
#else
#define COPY_POSITION(v)    gl_Position = gl_PositionIn[v]
#endif

////////////////////////////////////////////////////////////////////////

void main()
{
#ifdef PRIMITIVE_FILTERING
    if( (uPrimitiveFilter >= 0) && ( gl_PrimitiveIDIn != uPrimitiveFilter) )
        return;
#endif

    COPY_COLOR   (0);
    COPY_NORMAL  (0);
    COPY_TEXTURE (0);
    COPY_L1_COORD(0);
    COPY_POSITION(0);
    EmitVertex();
    
    COPY_COLOR   (1);
    COPY_NORMAL  (1);
    COPY_TEXTURE (1);
    COPY_L1_COORD(1);
    COPY_POSITION(1);
    EmitVertex();
    
    COPY_COLOR   (2);
    COPY_NORMAL  (2);
    COPY_TEXTURE (2);
    COPY_L1_COORD(2);
    COPY_POSITION(2);
    EmitVertex();
    EndPrimitive();
}